import sys
import os
import bpy
import numpy as np
from PyQt5.QtWidgets import (QApplication, QMainWindow, QPushButton, QLabel, 
                            QVBoxLayout, QHBoxLayout, QWidget, QFrame, QMenu, QAction)
from PyQt5.QtGui import QFont, QColor, QPalette, QIcon, QPixmap
from PyQt5.QtCore import Qt, QSize, QTimer

# Import PyOpenGL for 3D rendering
from OpenGL.GL import *
from OpenGL.GLU import *
from PyQt5.QtOpenGL import QGLWidget

class BlenderModelLoader:
    """Utility class to load and convert Blender models for OpenGL rendering"""
    
    @staticmethod
    def load_model(blend_file_path, object_name=None):
        """
        Load a model from a Blender file and extract mesh data
        Returns vertices, faces, and animation data
        """
        # This would normally use Blender Python API to load the model
        # For prototype purposes, we'll return simplified data
        
        print(f"Loading model from {blend_file_path}")
        
        # Check if it's Yoruichi or drumset model
        if "Yoruichi" in blend_file_path:
            return BlenderModelLoader._load_yoruichi_model()
        elif "Drumset" in blend_file_path:
            return BlenderModelLoader._load_drumset_model()
        else:
            print(f"Unknown model type: {blend_file_path}")
            return None
    
    @staticmethod
    def _load_yoruichi_model():
        """Return simplified Yoruichi model data"""
        # In a real implementation, this would extract actual mesh data from the Blender file
        
        # Simplified model data
        model_data = {
            'vertices': [
                # Body vertices (simplified)
                [0, 1.2, -0.5], [0.4, 1.2, -0.5], [0.4, 1.8, -0.5], [0, 1.8, -0.5],
                [0, 1.2, -0.7], [0.4, 1.2, -0.7], [0.4, 1.8, -0.7], [0, 1.8, -0.7],
                
                # Head vertices (simplified)
                [0, 1.9, -0.5], [0.2, 1.9, -0.5], [0.2, 2.1, -0.5], [0, 2.1, -0.5],
                [0, 1.9, -0.7], [0.2, 1.9, -0.7], [0.2, 2.1, -0.7], [0, 2.1, -0.7],
                
                # Arms vertices (simplified)
                [0.4, 1.6, -0.5], [0.6, 1.4, -0.5], [0.6, 1.5, -0.5], [0.4, 1.7, -0.5],
                [0.4, 1.6, -0.7], [0.6, 1.4, -0.7], [0.6, 1.5, -0.7], [0.4, 1.7, -0.7],
                
                [-0.2, 1.6, -0.5], [-0.4, 1.4, -0.5], [-0.4, 1.5, -0.5], [-0.2, 1.7, -0.5],
                [-0.2, 1.6, -0.7], [-0.4, 1.4, -0.7], [-0.4, 1.5, -0.7], [-0.2, 1.7, -0.7],
                
                # Legs vertices (simplified)
                [0.1, 1.2, -0.5], [0.3, 1.2, -0.5], [0.3, 0.8, -0.5], [0.1, 0.8, -0.5],
                [0.1, 1.2, -0.7], [0.3, 1.2, -0.7], [0.3, 0.8, -0.7], [0.1, 0.8, -0.7],
                
                [-0.1, 1.2, -0.5], [-0.3, 1.2, -0.5], [-0.3, 0.8, -0.5], [-0.1, 0.8, -0.5],
                [-0.1, 1.2, -0.7], [-0.3, 1.2, -0.7], [-0.3, 0.8, -0.7], [-0.1, 0.8, -0.7],
            ],
            'faces': [
                # Body faces
                [0, 1, 2, 3], [1, 5, 6, 2], [5, 4, 7, 6], [4, 0, 3, 7], [3, 2, 6, 7], [4, 5, 1, 0],
                
                # Head faces
                [8, 9, 10, 11], [9, 13, 14, 10], [13, 12, 15, 14], [12, 8, 11, 15], [11, 10, 14, 15], [12, 13, 9, 8],
                
                # Right arm faces
                [16, 17, 18, 19], [17, 21, 22, 18], [21, 20, 23, 22], [20, 16, 19, 23], [19, 18, 22, 23], [20, 21, 17, 16],
                
                # Left arm faces
                [24, 25, 26, 27], [25, 29, 30, 26], [29, 28, 31, 30], [28, 24, 27, 31], [27, 26, 30, 31], [28, 29, 25, 24],
                
                # Right leg faces
                [32, 33, 34, 35], [33, 37, 38, 34], [37, 36, 39, 38], [36, 32, 35, 39], [35, 34, 38, 39], [36, 37, 33, 32],
                
                # Left leg faces
                [40, 41, 42, 43], [41, 45, 46, 42], [45, 44, 47, 46], [44, 40, 43, 47], [43, 42, 46, 47], [44, 45, 41, 40],
            ],
            'colors': [
                # Body color (purple)
                [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7],
                
                # Head color (purple)
                [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7],
                
                # Right arm color (purple)
                [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7],
                
                # Left arm color (purple)
                [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7],
                
                # Right leg color (purple)
                [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7],
                
                # Left leg color (purple)
                [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7], [0.6, 0.3, 0.7],
            ],
            'animations': {
                'idle': {
                    'frames': 30,
                    'keyframes': {
                        0: {'position': [0, 0, 0], 'rotation': [0, 0, 0]},
                        15: {'position': [0, 0.05, 0], 'rotation': [0, 0, 0]},
                        30: {'position': [0, 0, 0], 'rotation': [0, 0, 0]},
                    }
                },
                'drumming': {
                    'frames': 20,
                    'keyframes': {
                        0: {'position': [0, 0, 0], 'rotation': [0, 0, 0]},
                        5: {'position': [0, 0, 0], 'rotation': [5, 0, 0]},
                        10: {'position': [0, 0, 0], 'rotation': [0, 0, 0]},
                        15: {'position': [0, 0, 0], 'rotation': [-5, 0, 0]},
                        20: {'position': [0, 0, 0], 'rotation': [0, 0, 0]},
                    }
                }
            }
        }
        
        return model_data
    
    @staticmethod
    def _load_drumset_model():
        """Return simplified drumset model data"""
        # In a real implementation, this would extract actual mesh data from the Blender file
        
        # Simplified model data for the drumset
        model_data = {
            'components': {
                'kick': {
                    'position': [0, -0.2, 0],
                    'scale': [0.6, 0.4, 0.4],
                    'color': [0.8, 0.1, 0.1],  # Red
                    'shape': 'cylinder',
                    'animation': {
                        'hit': {
                            'frames': 10,
                            'keyframes': {
                                0: {'position': [0, 0, 0]},
                                5: {'position': [0, 0, 0.05]},
                                10: {'position': [0, 0, 0]},
                            }
                        }
                    }
                },
                'snare': {
                    'position': [0.5, 0, 0.3],
                    'scale': [0.25, 0.15, 0.25],
                    'color': [0.9, 0.9, 0.9],  # White
                    'shape': 'cylinder',
                    'animation': {
                        'hit': {
                            'frames': 10,
                            'keyframes': {
                                0: {'position': [0, 0, 0]},
                                5: {'position': [0, -0.05, 0]},
                                10: {'position': [0, 0, 0]},
                            }
                        }
                    }
                },
                'hihat': {
                    'position': [0.8, 0.3, 0.3],
                    'scale': [0.3, 0.01, 0.3],
                    'color': [0.8, 0.8, 0.2],  # Gold
                    'shape': 'cylinder',
                    'animation': {
                        'hit': {
                            'frames': 10,
                            'keyframes': {
                                0: {'position': [0, 0, 0], 'separation': 0.0},
                                5: {'position': [0, 0, 0], 'separation': 0.1},
                                10: {'position': [0, 0, 0], 'separation': 0.0},
                            }
                        }
                    }
                },
                'tom1': {
                    'position': [0.3, 0.2, -0.2],
                    'scale': [0.2, 0.15, 0.2],
                    'color': [0.1, 0.1, 0.8],  # Blue
                    'shape': 'cylinder',
                    'animation': {
                        'hit': {
                            'frames': 10,
                            'keyframes': {
                                0: {'position': [0, 0, 0]},
                                5: {'position': [0, -0.05, 0]},
                                10: {'position': [0, 0, 0]},
                            }
                        }
                    }
                },
                'tom2': {
                    'position': [-0.3, 0.2, -0.2],
                    'scale': [0.2, 0.15, 0.2],
                    'color': [0.1, 0.1, 0.8],  # Blue
                    'shape': 'cylinder',
                    'animation': {
                        'hit': {
                            'frames': 10,
                            'keyframes': {
                                0: {'position': [0, 0, 0]},
                                5: {'position': [0, -0.05, 0]},
                                10: {'position': [0, 0, 0]},
                            }
                        }
                    }
                },
                'crash': {
                    'position': [0.7, 0.5, -0.3],
                    'scale': [0.35, 0.01, 0.35],
                    'color': [0.8, 0.8, 0.2],  # Gold
                    'shape': 'cylinder',
                    'animation': {
                        'hit': {
                            'frames': 10,
                            'keyframes': {
                                0: {'rotation': [0, 0, 0]},
                                5: {'rotation': [0, 0, 5]},
                                10: {'rotation': [0, 0, 0]},
                            }
                        }
                    }
                },
                'ride': {
                    'position': [-0.7, 0.5, -0.3],
                    'scale': [0.4, 0.01, 0.4],
                    'color': [0.8, 0.8, 0.2],  # Gold
                    'shape': 'cylinder',
                    'animation': {
                        'hit': {
                            'frames': 10,
                            'keyframes': {
                                0: {'rotation': [0, 0, 0]},
                                5: {'rotation': [0, 0, 5]},
                                10: {'rotation': [0, 0, 0]},
                            }
                        }
                    }
                },
                'left_stick': {
                    'position': [0.5, 0.3, 0.5],
                    'scale': [0.02, 0.02, 0.4],
                    'color': [0.8, 0.6, 0.4],  # Wood color
                    'shape': 'cube',
                    'animation': {
                        'hit': {
                            'frames': 10,
                            'keyframes': {
                                0: {'rotation': [0, 0, 0]},
                                5: {'rotation': [-30, 0, 0]},
                                10: {'rotation': [0, 0, 0]},
                            }
                        }
                    }
                },
                'right_stick': {
                    'position': [-0.5, 0.3, 0.5],
                    'scale': [0.02, 0.02, 0.4],
                    'color': [0.8, 0.6, 0.4],  # Wood color
                    'shape': 'cube',
                    'animation': {
                        'hit': {
                            'frames': 10,
                            'keyframes': {
                                0: {'rotation': [0, 0, 0]},
                                5: {'rotation': [-30, 0, 0]},
                                10: {'rotation': [0, 0, 0]},
                            }
                        }
                    }
                }
            }
        }
        
        return model_data

class ModelRenderer(QGLWidget):
    def __init__(self, parent=None):
        super(ModelRenderer, self).__init__(parent)
        self.setMinimumSize(800, 600)
        self.rotation = 0
        
        # Load models
        project_dir = "/home/ubuntu/project"
        self.yoruichi_model = BlenderModelLoader.load_model(
            os.path.join(project_dir, "models/yoruichi/YoruichiPhase1_Rigged.blend")
        )
        self.drumset_model = BlenderModelLoader.load_model(
            os.path.join(project_dir, "models/drumset/Drumset_Rigged.blend")
        )
        
        # Animation state
        self.animations = {
            'kick': {'active': False, 'frame': 0, 'max_frames': 10},
            'snare': {'active': False, 'frame': 0, 'max_frames': 10},
            'hihat': {'active': False, 'frame': 0, 'max_frames': 10},
            'tom1': {'active': False, 'frame': 0, 'max_frames': 10},
            'tom2': {'active': False, 'frame': 0, 'max_frames': 10},
            'crash': {'active': False, 'frame': 0, 'max_frames': 10},
            'ride': {'active': False, 'frame': 0, 'max_frames': 10},
            'yoruichi': {'active': False, 'frame': 0, 'max_frames': 20, 'animation': 'idle'}
        }
        
        # Start animation timer
        self.timer = QTimer(self)
        self.timer.timeout.connect(self.update_animation)
        self.timer.start(16)  # ~60fps
    
    def initializeGL(self):
        glClearColor(0.2, 0.2, 0.2, 1.0)
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_LIGHTING)
        glEnable(GL_LIGHT0)
        glEnable(GL_COLOR_MATERIAL)
        
        # Set up light
        glLightfv(GL_LIGHT0, GL_POSITION, [0, 5, 5, 1])
        glLightfv(GL_LIGHT0, GL_AMBIENT, [0.5, 0.5, 0.5, 1])
        glLightfv(GL_LIGHT0, GL_DIFFUSE, [1, 1, 1, 1])
    
    def resizeGL(self, width, height):
        glViewport(0, 0, width, height)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(45, width / height, 0.1, 100.0)
        glMatrixMode(GL_MODELVIEW)
    
    def paintGL(self):
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        glLoadIdentity()
        
        # Position camera
        gluLookAt(0, 2, 5,  # Eye position
                 0, 0, 0,   # Look at point
                 0, 1, 0)   # Up vector
        
        # Rotate scene
        glRotatef(self.rotation, 0, 1, 0)
        
        # Draw Yoruichi
        self.draw_yoruichi()
        
        # Draw drumset
        self.draw_drumset()
    
    def draw_yoruichi(self):
        if not self.yoruichi_model:
            return
        
        # Get animation state
        anim_state = self.animations['yoruichi']
        
        # Draw Yoruichi using the model data
        glPushMatrix()
        
        # Apply animation if active
        if anim_state['active'] and anim_state['animation'] in self.yoruichi_model['animations']:
            animation = self.yoruichi_model['animations'][anim_state['animation']]
            frame = anim_state['frame']
            max_frames = animation['frames']
            
            # Find appropriate keyframes
            prev_keyframe = 0
            next_keyframe = max_frames
            
            for keyframe in sorted(animation['keyframes'].keys()):
                if keyframe <= frame:
                    prev_keyframe = keyframe
                if keyframe > frame and keyframe < next_keyframe:
                    next_keyframe = keyframe
            
            # Interpolate between keyframes
            if prev_keyframe != next_keyframe:
                prev_data = animation['keyframes'][prev_keyframe]
                next_data = animation['keyframes'][next_keyframe]
                
                t = (frame - prev_keyframe) / (next_keyframe - prev_keyframe)
                
                # Apply interpolated position
                if 'position' in prev_data and 'position' in next_data:
                    pos = [
                        prev_data['position'][0] + t * (next_data['position'][0] - prev_data['position'][0]),
                        prev_data['position'][1] + t * (next_data['position'][1] - prev_data['position'][1]),
                        prev_data['position'][2] + t * (next_data['position'][2] - prev_data['position'][2])
                    ]
                    glTranslatef(pos[0], pos[1], pos[2])
                
                # Apply interpolated rotation
                if 'rotation' in prev_data and 'rotation' in next_data:
                    rot = [
                        prev_data['rotation'][0] + t * (next_data['rotation'][0] - prev_data['rotation'][0]),
                        prev_data['rotation'][1] + t * (next_data['rotation'][1] - prev_data['rotation'][1]),
                        prev_data['rotation'][2] + t * (next_data['rotation'][2] - prev_data['rotation'][2])
                    ]
                    glRotatef(rot[0], 1, 0, 0)
                    glRotatef(rot[1], 0, 1, 0)
                    glRotatef(rot[2], 0, 0, 1)
        
        # Draw the model
        glBegin(GL_QUADS)
        for i, face in enumerate(self.yoruichi_model['faces']):
            color = self.yoruichi_model['colors'][i]
            glColor3f(color[0], color[1], color[2])
            
            for vertex_idx in face:
                vertex = self.yoruichi_model['vertices'][vertex_idx]
                glVertex3f(vertex[0], vertex[1], vertex[2])
        glEnd()
        
        glPopMatrix()
    
    def draw_drumset(self):
        if not self.drumset_model:
            return
        
        # Base/floor
        glPushMatrix()
        glTranslatef(0, -0.5, 0)
        glScalef(2, 0.1, 2)
        glColor3f(0.3, 0.3, 0.3)
        self.draw_cube()
        glPopMatrix()
        
        # Draw each drum component
        for component_name, component in self.drumset_model['components'].items():
            glPushMatrix()
            
            # Get base position and scale
            pos = component['position']
            scale = component['scale']
            color = component['color']
            
            # Apply animation if active
            if component_name in self.animations and self.animations[component_name]['active']:
                anim = self.animations[component_name]
                frame = anim['frame']
                max_frames = anim['max_frames']
                
                # Get animation data from model
                if 'animation' in component and 'hit' in component['animation']:
                    hit_anim = component['animation']['hit']
                    
                    # Find appropriate keyframes
                    prev_keyframe = 0
                    next_keyframe = max_frames
                    
                    for keyframe in sorted(hit_anim['keyframes'].keys()):
                        if keyframe <= frame:
                            prev_keyframe = keyframe
                        if keyframe > frame and keyframe < next_keyframe:
                            next_keyframe = keyframe
                    
                    # Interpolate between keyframes
                    if prev_keyframe != next_keyframe:
                        prev_data = hit_anim['keyframes'][prev_keyframe]
                        next_data = hit_anim['keyframes'][next_keyframe]
                        
                        t = (frame - prev_keyframe) / (next_keyframe - prev_keyframe)
                        
                        # Apply interpolated position
                        if 'position' in prev_data and 'position' in next_data:
                            offset = [
                                prev_data['position'][0] + t * (next_data['position'][0] - prev_data['position'][0]),
                                prev_data['position'][1] + t * (next_data['position'][1] - prev_data['position'][1]),
                                prev_data['position'][2] + t * (next_data['position'][2] - prev_data['position'][2])
                            ]
                            pos = [pos[0] + offset[0], pos[1] + offset[1], pos[2] + offset[2]]
                        
                        # Apply interpolated rotation
                        if 'rotation' in prev_data and 'rotation' in next_data:
                            glTranslatef(pos[0], pos[1], pos[2])
                            
                            rot = [
                                prev_data['rotation'][0] + t * (next_data['rotation'][0] - prev_data['rotation'][0]),
                                prev_data['rotation'][1] + t * (next_data['rotation'][1] - prev_data['rotation'][1]),
                                prev_data['rotation'][2] + t * (next_data['rotation'][2] - prev_data['rotation'][2])
                            ]
                            glRotatef(rot[0], 1, 0, 0)
                            glRotatef(rot[1], 0, 1, 0)
                            glRotatef(rot[2], 0, 0, 1)
                            
                            # Special case for hi-hat
                            if component_name == 'hihat' and 'separation' in prev_data and 'separation' in next_data:
                                separation = prev_data['separation'] + t * (next_data['separation'] - prev_data['separation'])
                                # Will handle this in the drawing code
                            
                            pos = [0, 0, 0]  # Already applied in the rotation
            
            # Apply position
            glTranslatef(pos[0], pos[1], pos[2])
            
            # Apply scale
            glScalef(scale[0], scale[1], scale[2])
            
            # Apply color
            glColor3f(color[0], color[1], color[2])
            
            # Draw the appropriate shape
            if component['shape'] == 'cylinder':
                self.draw_cylinder()
            else:
                self.draw_cube()
            
            glPopMatrix()
            
            # Special case for hi-hat - draw the top part
            if component_name == 'hihat' and 'hihat' in self.animations and self.animations['hihat']['active']:
                anim = self.animations['hihat']
                frame = anim['frame']
                max_frames = anim['max_frames']
                
                # Get animation data
                if 'animation' in component and 'hit' in component['animation']:
                    hit_anim = component['animation']['hit']
                    
                    # Find appropriate keyframes
                    prev_keyframe = 0
                    next_keyframe = max_frames
                    
                    for keyframe in sorted(hit_anim['keyframes'].keys()):
                        if keyframe <= frame:
                            prev_keyframe = keyframe
                        if keyframe > frame and keyframe < next_keyframe:
                            next_keyframe = keyframe
                    
                    # Interpolate between keyframes
                    if prev_keyframe != next_keyframe:
                        prev_data = hit_anim['keyframes'][prev_keyframe]
                        next_data = hit_anim['keyframes'][next_keyframe]
                        
                        t = (frame - prev_keyframe) / (next_keyframe - prev_keyframe)
                        
                        # Apply interpolated separation
                        if 'separation' in prev_data and 'separation' in next_data:
                            separation = prev_data['separation'] + t * (next_data['separation'] - prev_data['separation'])
                            
                            # Draw the top part of the hi-hat
                            glPushMatrix()
                            glTranslatef(pos[0], pos[1] + separation + 0.02, pos[2])
                            glScalef(scale[0], scale[1], scale[2])
                            glColor3f(color[0], color[1], color[2])
                            self.draw_cylinder()
                            glPopMatrix()
    
    def draw_cube(self):
        vertices = [
            # Front face
            [-1, -1, 1], [1, -1, 1], [1, 1, 1], [-1, 1, 1],
            # Back face
            [-1, -1, -1], [1, -1, -1], [1, 1, -1], [-1, 1, -1]
        ]
        
        faces = [
            [0, 1, 2, 3],  # Front
            [1, 5, 6, 2],  # Right
            [5, 4, 7, 6],  # Back
            [4, 0, 3, 7],  # Left
            [3, 2, 6, 7],  # Top
            [4, 5, 1, 0]   # Bottom
        ]
        
        normals = [
            [0, 0, 1],    # Front
            [1, 0, 0],    # Right
            [0, 0, -1],   # Back
            [-1, 0, 0],   # Left
            [0, 1, 0],    # Top
            [0, -1, 0]    # Bottom
        ]
        
        glBegin(GL_QUADS)
        for i, face in enumerate(faces):
            glNormal3fv(normals[i])
            for vertex in face:
                glVertex3fv(vertices[vertex])
        glEnd()
    
    def draw_cylinder(self, slices=20):
        # Draw a cylinder with top and bottom faces
        # Top face
        glBegin(GL_TRIANGLE_FAN)
        glNormal3f(0, 1, 0)
        glVertex3f(0, 1, 0)  # Center
        for i in range(slices + 1):
            angle = 2 * np.pi * i / slices
            x = np.cos(angle)
            z = np.sin(angle)
            glVertex3f(x, 1, z)
        glEnd()
        
        # Bottom face
        glBegin(GL_TRIANGLE_FAN)
        glNormal3f(0, -1, 0)
        glVertex3f(0, -1, 0)  # Center
        for i in range(slices + 1):
            angle = 2 * np.pi * i / slices
            x = np.cos(angle)
            z = -np.sin(angle)
            glVertex3f(x, -1, z)
        glEnd()
        
        # Side faces
        glBegin(GL_QUAD_STRIP)
        for i in range(slices + 1):
            angle = 2 * np.pi * i / slices
            x = np.cos(angle)
            z = np.sin(angle)
            glNormal3f(x, 0, z)
            glVertex3f(x, 1, z)
            glVertex3f(x, -1, z)
        glEnd()
    
    def update_animation(self):
        # Update rotation for continuous spinning
        self.rotation = (self.rotation + 0.5) % 360
        
        # Update animation frames
        for drum, anim in self.animations.items():
            if anim['active']:
                anim['frame'] += 1
                if anim['frame'] >= anim['max_frames']:
                    anim['frame'] = 0
                    if drum != 'yoruichi':  # Keep Yoruichi animating
                        anim['active'] = False
        
        self.update()
    
    def trigger_drum(self, drum_name):
        if drum_name in self.animations:
            self.animations[drum_name]['active'] = True
            self.animations[drum_name]['frame'] = 0
            
            # Also trigger Yoruichi's drumming animation
            self.animations['yoruichi']['active'] = True
            self.animations['yoruichi']['animation'] = 'drumming'
            self.animations['yoruichi']['frame'] = 0
    
    def mousePressEvent(self, event):
        # Map mouse clicks to drum hits
        x, y = event.x(), event.y()
        width, height = self.width(), self.height()
        
        # Simple mapping of screen regions to drums
        if y < height / 2:
            if x < width / 3:
                self.trigger_drum('crash')
            elif x < 2 * width / 3:
                self.trigger_drum('tom1')
            else:
                self.trigger_drum('ride')
        else:
            if x < width / 3:
                self.trigger_drum('hihat')
            elif x < 2 * width / 3:
                self.trigger_drum('kick')
            else:
                self.trigger_drum('snare')

class KrownedKueensDAW(QMainWindow):
    def __init__(self):
        super().__init__()
        
        # Set window title and size
        self.setWindowTitle("BLEACH: Krowned Kueens Digital Audio Workstation")
        self.setMinimumSize(1024, 768)
        
        # Set orange background color (#FFA500)
        palette = QPalette()
        palette.setColor(QPalette.Window, QColor("#FFA500"))
        self.setPalette(palette)
        
        # Create main widget and layout
        main_widget = QWidget()
        main_layout = QVBoxLayout(main_widget)
        
        # Create title label
        title_label = QLabel("Krowned Kueens Digital Audio Workstation")
        title_font = QFont("Arial", 24, QFont.Bold)
        title_label.setFont(title_font)
        title_label.setAlignment(Qt.AlignCenter)
        title_label.setStyleSheet("color: white; margin: 20px;")
        
        # Create menu frame
        menu_frame = QFrame()
        menu_frame.setFrameShape(QFrame.StyledPanel)
        menu_frame.setStyleSheet("background-color: rgba(255, 255, 255, 0.2); border-radius: 10px;")
        menu_layout = QVBoxLayout(menu_frame)
        
        # Create menu buttons
        menu_buttons = [
            "New Project", 
            "Load Project", 
            "Audio Settings", 
            "Drum Kits", 
            "Effects", 
            "Export"
        ]
        
        for button_text in menu_buttons:
            button = QPushButton(button_text)
            button.setMinimumHeight(50)
            button.setFont(QFont("Arial", 14))
            button.setStyleSheet("""
                QPushButton {
                    background-color: rgba(255, 255, 255, 0.3);
                    color: white;
                    border-radius: 5px;
                    padding: 10px;
                    text-align: left;
                    padding-left: 20px;
                }
                QPushButton:hover {
                    background-color: rgba(255, 255, 255, 0.5);
                }
                QPushButton:pressed {
                    background-color: rgba(255, 255, 255, 0.7);
                }
            """)
            menu_layout.addWidget(button)
        
        # Create start producing button
        self.start_button = QPushButton("Start Producing!")
        self.start_button.setMinimumHeight(70)
        self.start_button.setFont(QFont("Arial", 18, QFont.Bold))
        self.start_button.setStyleSheet("""
            QPushButton {
                background-color: #6A0DAD;
                color: white;
                border-radius: 10px;
                padding: 15px;
            }
            QPushButton:hover {
                background-color: #8A2BE2;
            }
            QPushButton:pressed {
                background-color: #4B0082;
            }
        """)
        self.start_button.clicked.connect(self.start_producing)
        
        # Create model renderer (initially hidden)
        self.model_renderer = ModelRenderer()
        self.model_renderer.hide()
        
        # Add widgets to main layout
        main_layout.addWidget(title_label)
        main_layout.addWidget(menu_frame)
        main_layout.addWidget(self.model_renderer)
        main_layout.addStretch()
        main_layout.addWidget(self.start_button)
        main_layout.addSpacing(30)
        
        # Set main widget
        self.setCentralWidget(main_widget)
        
        # Create menu bar
        menubar = self.menuBar()
        menubar.setStyleSheet("background-color: #FF8C00; color: white;")
        
        # File menu
        file_menu = menubar.addMenu('File')
        file_menu.setStyleSheet("background-color: #FF8C00;")
        
        new_action = QAction('New Project', self)
        open_action = QAction('Open Project', self)
        save_action = QAction('Save Project', self)
        exit_action = QAction('Exit', self)
        exit_action.triggered.connect(self.close)
        
        file_menu.addAction(new_action)
        file_menu.addAction(open_action)
        file_menu.addAction(save_action)
        file_menu.addSeparator()
        file_menu.addAction(exit_action)
        
        # Edit menu
        edit_menu = menubar.addMenu('Edit')
        
        # View menu
        view_menu = menubar.addMenu('View')
        
        # Help menu
        help_menu = menubar.addMenu('Help')
        
        # Production mode flag
        self.production_mode = False
    
    def start_producing(self):
        if not self.production_mode:
            # Switch to production mode
            self.production_mode = True
            self.start_button.setText("Stop Producing")
            
            # Show the model renderer
            self.model_renderer.show()
            
            # Hide the menu frame
            for i in range(1, self.centralWidget().layout().count() - 3):
                widget = self.centralWidget().layout().itemAt(i).widget()
                if isinstance(widget, QFrame):
                    widget.hide()
            
            # Start Yoruichi's idle animation
            self.model_renderer.animations['yoruichi']['active'] = True
            self.model_renderer.animations['yoruichi']['animation'] = 'idle'
            self.model_renderer.animations['yoruichi']['frame'] = 0
        else:
            # Switch back to menu mode
            self.production_mode = False
            self.start_button.setText("Start Producing!")
            
            # Hide the model renderer
            self.model_renderer.hide()
            
            # Show the menu frame
            for i in range(1, self.centralWidget().layout().count() - 3):
                widget = self.centralWidget().layout().itemAt(i).widget()
                if isinstance(widget, QFrame):
                    widget.show()

def main():
    app = QApplication(sys.argv)
    window = KrownedKueensDAW()
    window.show()
    sys.exit(app.exec_())

if __name__ == "__main__":
    main()
